@@ -159,9 +159,9 @@ RegisterAuthorizations (void)
 XID
 CheckAuthorization (
     unsigned int name_length,
-    char	*name,
+    const char	*name,
     unsigned int data_length,
-    char	*data,
+    const char	*data,
     ClientPtr client,
     char	**reason)	/* failure message.  NULL for default msg */
 {
@@ -256,9 +256,9 @@ AuthorizationFromID (
 int
 RemoveAuthorization (
 	unsigned short	name_length,
-	char		*name,
+	const char	*name,
 	unsigned short	data_length,
-	char		*data)
+	const char	*data)
 {
     int	i;
 
@@ -274,7 +274,8 @@ RemoveAuthorization (
 }
 
 int
-AddAuthorization (unsigned name_length, char *name, unsigned data_length, char *data)
+AddAuthorization (unsigned name_length, const char *name,
+		  unsigned data_length, char *data)
 {
     int	i;
 
@@ -294,9 +295,9 @@ AddAuthorization (unsigned name_length, char *name, unsigned data_length, char *
 XID
 GenerateAuthorization(
 	unsigned name_length,
-	char	*name,
+	const char	*name,
 	unsigned data_length,
-	char	*data,
+	const char	*data,
 	unsigned *data_length_return,
 	char	**data_return)
 {
@@ -314,8 +315,6 @@ GenerateAuthorization(
     return -1;
 }
 
-#ifdef HAVE_URANDOM
-
 void
 GenerateRandomData (int len, char *buf)
 {
@@ -326,45 +325,4 @@ GenerateRandomData (int len, char *buf)
     close(fd);
 }
 
-#else /* !HAVE_URANDOM */
-
-/* A random number generator that is more unpredictable
-   than that shipped with some systems.
-   This code is taken from the C standard. */
-
-static unsigned long int next = 1;
-
-static int
-xdm_rand(void)
-{
-    next = next * 1103515245 + 12345;
-    return (unsigned int)(next/65536) % 32768;
-}
-
-static void
-xdm_srand(unsigned int seed)
-{
-    next = seed;
-}
-
-void
-GenerateRandomData (int len, char *buf)
-{
-    static int seed;
-    int value;
-    int i;
-
-    seed += GetTimeInMillis();
-    xdm_srand (seed);
-    for (i = 0; i < len; i++)
-    {
-	value = xdm_rand ();
-	buf[i] ^= (value & 0xff00) >> 8;
-    }
-
-    /* XXX add getrusage, popen("ps -ale") */
-}
-
-#endif /* HAVE_URANDOM */
-
 #endif /* XCSECURITY */
